
function [gx, gu, Hxx, Hxu, Huu] = Durable_model

%Inputs: none
%
%Output: Analytical first derivatives of the constraint function g and
%second derivatives of the Hamiltonian H
%
%Calls: anal_derivative.m 
%

%Define parameters
syms THETA R BETTA GAMA delta phik phib ybar bbar rho1 rho2 sigma1 sigma2 sigmaz 

%Define model variables
syms c k b yz Inv y1 y2  

%Define the vector of controls u, and states x
u = [c; Inv]; % Column control vector 
x = [y1; y2; yz; k; b];  %Column state vector
nx =length(x);  %number of states

%Define the vector of Lagrange multipliers for the state x
syms lambdaSS [nx 1];
  
%Define objective function

f = (c^THETA*k^(1-THETA))^(1-GAMA)/(1-GAMA)-0.5*phib*(b-bbar)^2;

%Define constraints
g1 = rho1*y1; %Motion of income shock
g2 = rho2*y2; %Motion of income shock
g3 = 0;
g4 = (1-delta)*k + Inv; %Motion of capital
g5 = R*b + ybar*exp(y1+y2+yz)-c-Inv-0.5*phik*k*(Inv/k-delta)^2; %Motion of wealth
g = [g1; g2; g3; g4;g5];

%Compute analytical derivatives of f
[gx, gu, Hxx, Hxu,Huu] = anal_derivative(f, g, x, u, lambdaSS);
