
% 5.5 Durable and Nondurable Consumption

% Copywrite (C) Written by Bo Zhang, zhangbo@swufe.edu.cn
% 
%----------------------------------------------------------------
% 0. Housekeeping (close all graphic windows)
%----------------------------------------------------------------

%close all;

%----------------------------------------------------------------
% 1. Defining variables
%----------------------------------------------------------------

var b c k i lamda q y1 y2;
varexo e_y1 e_y2 e_z;
parameters  beta gamma theta delta R phik  phib  ybar  ;

%----------------------------------------------------------------
% 2. Calibration
%----------------------------------------------------------------
steady_state_model;
R=1.01;
beta=1/R;
delta=0.02;
gamma=2;
theta=0.8;
phik=0.5;
phib=0.001;
ybar=1;

b=log(0.2);
c=log(((R-1)*exp(b)+1)/(1+delta*beta*(1-theta)/(theta*(1-beta*(1-delta)))));
k=log(beta*(1-theta)/(theta*(1-beta*(1-delta)))*exp(c));
%lamdabar=(cbar^theta*kbar^(1-theta))^(-gamma)*theta*cbar^(theta-1)*kbar^(1-theta);
lamda=log(exp(c)^(theta*(1-gamma))*exp(k)^(-gamma*(1-theta)-theta)*(1-theta)/(1/beta+delta-1));
i=log(delta*exp(k));
q=lamda;
y1=0;
y2=0;

end;

%----------------------------------------------------------------
% 3. Model
%----------------------------------------------------------------

model; 
%Foc--- c
exp(lamda)=(exp(c)^theta*(exp(k(-1)))^(1-theta))^(-gamma)*theta*exp(c)^(theta-1)*(exp(k(-1)))^(1-theta);

%Foc---i
exp(q)=exp(lamda)+exp(lamda)*phik*(exp(i)/exp(k(-1))-delta);

%Foc---k
exp(q)=beta*((exp(c(+1))^theta*exp(k)^(1-theta))^(-gamma)*exp(c(+1))^theta*exp(k)^(-theta)*(1-theta)+exp(q(+1))*(1-delta)-exp(lamda(+1))*(phik/2*((exp(i(+1))/exp(k))-delta)^2+phik*exp(k)*(exp(i(+1))/exp(k)-delta)*exp(i(+1))*(-1/exp(k)^2)));

%Foc---b
exp(lamda)=beta*exp(lamda(+1))*R-beta*phib*(exp(b)-0.2);

%law of motion of k
exp(k)=(1-delta)*exp(k(-1))+exp(i);

%resource constraint
exp(c)+exp(b)+exp(i)+phik/2*exp(k(-1))*((exp(i)/exp(k(-1))-delta)^2)=R*exp(b(-1))+exp(y1+y2+e_z);

%%%shocks
y1=0.97*y1(-1)+e_y1;
y2=0.8*y2(-1)+e_y2;
end;

steady;
%----------------------------------------------------------------
% 4. Computation
%----------------------------------------------------------------
shocks;

var e_y1=1;
var e_y2=1;
var e_z=1;
end;
stoch_simul(irf=50,order=1)c  i k b;