%Solving a simple RBC model with preference and technology shocks and with
%Capital utilitzation
%State vector x(t) =(a1(t),a2(t),K(t))', control vector u(t)=C(t)'
%Call codes Pitfall_model.m and Pitfall_model_ss.m


clear all;
close all;
tic


[gx, gu, Hxx, Hxu, Huu] = Pitfall_model;

%Numerical Evaluation
%Steady State and Parameter Values
[ALFA,BETTA, rho1, rho2, sigma1,sigma2, c,k, a1, a2, lambdaSS1,lambdaSS2,lambdaSS3] = Pitfall_model_ss;

%Obtain numerical derivatives of f
num_derivative

%% Solution under Full information 
%% Using MWY(2022) Notation

Q = -nHxx/2;
S = -nHxu/2;
R = -nHuu/2;
A = ngx;
B = ngu;

nx =size(Q,1);

P0 = eye(nx,nx);
[P,F,err,iter] = LQG(A,B,S,Q,R,BETTA,P0); % Call a function to compute Riccati equation
disp('Linear decision rule coefficents u(t) =-F*\hat{x}(t) : ')
F

%RI solution
L = zeros(3,2);
L(1,1) = sigma1;
L(2,2) = sigma2;
W = L*L';

lambda=  0.005; %lambdav(1);
Omega = F'*(R+ BETTA*B'*P*B)*F;

    Sigma_pr0 = A*A'+W; %Initial guess of optimal prior covariance matrix 
    
    Theta0 = Omega; % Initial guess of dynamic benefit matrix
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,BETTA,A,W,Omega, Theta0,Sigma_pr0); 

[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Compute signal structure

display('Steady state signal structure: [C V], Sigma')
[C,V]
SigmaSS


%impulse responses to one standard deviation innovation shock

Period = 40;
[X_full, u_full, X, u] = RI_IRF3(A,B,F,L, Period, SigmaSS, C, V);

toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Transform to log linear responses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure;
time =1:Period;
%set(gcf,'defaulttextinterpreter','latex')
subplot(2,2,1)
plot(time,100*u_full(1,:,1)/c,'LineWidth',1.5);
    hold on
plot(time,100*u(1,:,1)/c,'--','LineWidth',1.5);
title('IRF of C_t to a_{1t} shock'); %,'interpreter','latex');
lgd = legend('Full Info','RI')
lgd.FontSize = 4;
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;


subplot(2,2,3)
plot(time,100*u_full(1,:,2)/c,'LineWidth',1.5);
    hold on
plot(time,100*u(1,:,2)/c,'--','LineWidth',1.5);
xlabel('time','Fontsize',7,'interpreter','latex');
lgd = legend('Full Info','RI')
lgd.FontSize = 4;
title('IRF of C_t to a_{2t} shock'); %,'interpreter','latex');
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Ad hoc LQ approximation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ConsPitfall

fon= 4; %FONT SIZE
time=1:Period;

subplot(2,2,2)
c1_imp = plot(time, u_full(1,:,1)/c*100, time,u1(1,:,1)/c*100,'-.b');
title('IRF of C_t to a_{1t} shock'); %,'interpreter','latex');
set(c1_imp,'LineWidth',1.5);
%xlabel('Periods');
%ylabel('(%)','Rotation',0);
lgd=legend('Full Infor','RI','Location','southeast');
lgd.FontSize = fon;
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,2,4)
c2_imp = plot(time, u_full(1,:,2)/c*100, time,u1(1,:,2)/c*100,'-.b');
title('IRF of C_t to a_{2t} shock'); %,'interpreter','latex');
set(c2_imp,'LineWidth',1.5);
xlabel('time','Fontsize',7,'interpreter','latex');
%ylabel('(%)','Rotation',0);
lgd=legend('Full Infor','RI','Location','southeast');
lgd.FontSize=fon;

ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

%print -depsc FigPitfall.eps