%Solving a simple RBC model with IST and TFP shocks
%State vector x(t) =(z(t),a(t),K(t))', control vector u(t)=(C(t),N(t),e(t))'
%Call codes RBCIST_model.m and RBCIST_model_ss.m


clear all;
close all;
tic
flag = 0; % flag = 0 means endogenous capital utilization; otherwise exogenous utilization e = 1.

[gx, gu, Hxx, Hxu, Huu] = RBCIST_model;

%Numerical Evaluation
%Steady State and Parameter Values
[ALFA,BETTA,delta,chi,nu, phi, gama, rho, rho_z, sigma,sigma_z, c,k,n,Inv,e, a,...
    z, lambdaSS1,lambdaSS2,lambdaSS3] = RBCIST_model_ss;

%Obtain numerical derivatives of f
num_derivative

%% Solution under Full information 
%% Using MWY(2022) Notation

Q = -nHxx/2;
S = -nHxu/2;
R = -nHuu/2;
A = ngx;
B = ngu;

nx =size(Q,1);

P0 = eye(nx,nx);
[P,F,err,iter] = LQG(A,B,S,Q,R,BETTA,P0); % Call a function to compute Riccati equation
disp('Linear decision rule coefficents: ')
% u = -F*\hat{x}; 
F

%RI solution
L = zeros(3,2);
L(1,1) = sigma_z;
L(2,2) = sigma;
W = L*L';

lambda=  0.002; %lambdav(1);
Omega = F'*(R+ BETTA*B'*P*B)*F;

    Sigma_pr0 = A*A'+W; %Initial guess of optimal prior covariance matrix 
    
    Theta0 = Omega; % Initial guess of dynamic benefit matrix
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,BETTA,A,W,Omega, Theta0,Sigma_pr0); 

[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Compute signal structure

display('Steady state signal structure: [C V], Sigma')
[C,V]
SigmaSS


%impulse responses to one standard deviation innovation shock

Period = 40;
[X_full, u_full, X, u] = RI_IRF3(A,B,F,L, Period, SigmaSS, C, V);

toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Transform to log linear responses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure;
time =1:Period;

subplot(2,4,1)
plot(time,100*u_full(1,:,1)/c,'LineWidth',1.5);
    hold on
plot(time,100*u(1,:,1)/c,'--','LineWidth',1.5);
title(['C_t to IST shock'])
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,4,2)
plot(time,100*u_full(2,:,1)/n,'LineWidth',1.5);
    hold on
plot(time,100*u(2,:,1)/n,'--','LineWidth',1.5);
title('N_t to TFP shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

%Compute I_t IRF
if flag ==0; %Full information case with endogenous capital utilization
Yt_full = k^ALFA*n^(1-ALFA)*X_full(2,:,1) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X_full(3,:,1)...
      + ALFA*k^ALFA*n^(1-ALFA)*u_full(3,:,1) + (1-ALFA)*k^ALFA*n^(-ALFA)*u_full(2,:,1);
 else %Full information case with exogenous capital utilization u =1
Yt_full = k^ALFA*n^(1-ALFA)*X_full(2,:,1) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X_full(3,:,1)...
     + (1-ALFA)*k^ALFA*n^(-ALFA)*u_full(2,:,1);
end

ItFull_TFP = Yt_full - u_full(1,:,1);

if flag ==0 %RI case with endogenous capital utilization
 Yt = k^ALFA*n^(1-ALFA)*X(2,:,1) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X(3,:,1)...
    + ALFA*k^ALFA*n^(1-ALFA)*u(3,:,1)+(1-ALFA)*k^ALFA*n^(-ALFA)*u(2,:,1);
 else %RI case with exogenous capital utilization = 1
 Yt = k^ALFA*n^(1-ALFA)*X(2,:,1) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X(3,:,1)...
    +(1-ALFA)*k^ALFA*n^(-ALFA)*u(2,:,1); 
end

ItRI_TFP =  Yt - u(1,:,1);

subplot(2,4,3)
plot(time,100*ItFull_TFP/Inv, 'LineWidth',1.5);
    hold on
plot(time,100*ItRI_TFP/Inv,'--','LineWidth',1.5);
title('I_t to IST shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

if flag ==0
subplot(2,4,4) %capital utilization
plot(time,100*u_full(3,:,1), 'LineWidth',1.5);
    hold on
plot(time,100*u(3,:,1),'--','LineWidth',1.5);
title('e_t to IST shock')
else 
subplot(2,4,4) %capital utilization
plot(time,ones(1,Period), 'LineWidth',1.5);
end
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,4,5)
plot(time,100*u_full(1,:,2)/c,'LineWidth',1.5);
    hold on
plot(time,100*u(1,:,2)/c,'--','LineWidth',1.5);
xlabel('time')
%legend('Full Info','RI')
title('C_t to TFP shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,4,6)
plot(time,100*u_full(2,:,2)/n,'LineWidth',1.5);
    hold on
plot(time,100*u(2,:,2)/n,'--','LineWidth',1.5);
xlabel('time')
%legend('Full Info','RI')
title('N_t to TFP shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

if flag ==0 %Full information case with endogenous capital utilization 
Yt_full = k^ALFA*n^(1-ALFA)*X_full(2,:,2) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X_full(3,:,2)...
    + ALFA*k^ALFA*n^(1-ALFA)*u_full(3,:,2) + (1-ALFA)*k^ALFA*n^(-ALFA)*u_full(2,:,2);
else %Full information case with exogenous capital utilization u =1
Yt_full = k^ALFA*n^(1-ALFA)*X_full(2,:,2) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X_full(3,:,2)...
     + (1-ALFA)*k^ALFA*n^(-ALFA)*u_full(2,:,2);
end
ItFull_news = Yt_full - u_full(1,:,2);

if flag ==0 %RI case with endogenous capital utilization
Yt = k^ALFA*n^(1-ALFA)*X(2,:,2) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X(3,:,2)...
    + ALFA*k^ALFA*n^(1-ALFA)*u(3,:,2)+(1-ALFA)*k^ALFA*n^(-ALFA)*u(2,:,2); 
else %RI case with exogenous capital utilization u =1
Yt = k^ALFA*n^(1-ALFA)*X(2,:,2) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X(3,:,2)...
    +(1-ALFA)*k^ALFA*n^(-ALFA)*u(2,:,2);
end

ItRI_news =  Yt - u(1,:,2);

subplot(2,4,7)
plot(time,100*ItFull_news/Inv,'LineWidth',1.5);
    hold on
plot(time,100*ItRI_news/Inv,'--','LineWidth',1.5);
xlabel('time')
title('I_t to TFP shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;


subplot(2,4,8) %capital utilization

if flag ==0
    plot(time,100*u_full(3,:,2), 'LineWidth',1.5);
        hold on
    plot(time,100*u(3,:,2),'--','LineWidth',1.5);
  else
    plot(time, ones(1,Period), 'LineWidth', 1.5);    
end
    
title('e_t to TFP shock')
lgd = legend('Full Info','RI')
lgd.FontSize = 4;
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

print -depsc FigRBCIST.eps