%Solving a simple RBC model with preference shock
%State vector x(t) =(z(t),a(t),K(t))', control vector u(t)=(I(t),N(t))'
%Call codes RBC_model.m and RBC_model_ss.m

clear all;
%close all;

[gx, gu, Hxx, Hxu, Huu] = RBC_model;

%Numerical Evaluation
%Steady State and Parameter Values
[ALFA,BETTA,delta,chi,nu,rho, rho_z, sigma,sigma_z, c,k,n,Inv,a, z, lambdaSS1,lambdaSS2,lambdaSS3] = RBC_model_ss;

%Obtain numerical derivatives of f
num_derivative

%% Solution under Full information 
%% Using MWY(2022) Notation

Q = -nHxx/2;
S = -nHxu/2;
R = -nHuu/2;
A = ngx;
B = ngu;

nx =size(Q,1);

P0 = eye(nx,nx);
[P,F,err,iter] = LQG(A,B,S,Q,R,BETTA,P0); % Call a function to compute Riccati equation
disp('Linear decision rule coefficents: ')
% u = -F*\hat{x}; u=[Inv;n], x = [z;a;k] 
F

%RI solution
L = zeros(3,2);
L(1,1) = sigma_z;
L(2,2) = sigma;
W = L*L';

lambda=  0.005; %lambdav(1);
Omega = F'*(R+ BETTA*B'*P*B)*F;

    Sigma_pr0 = A*A'+W; %Initial guess of optimal prior covariance matrix 
    
    Theta0 = Omega; % Initial guess of dynamic benefit matrix
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,BETTA,A,W,Omega, Theta0,Sigma_pr0); 

[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Compute signal structure

display('Steady state signal structure: [C V], Sigma')
[C,V]
SigmaSS


%impulse responses to one standard deviation innovation shock

Period = 40;
[X_full, u_full, X, u,Sigmax,Sigmau] = RI_IRF3(A,B,F,L, Period, SigmaSS, C, V,0);

Sigmax
Sigmau

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Transform to log linear responses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure;
time =1:Period;
subplot(2,3,1)
plot(time,100*u_full(1,:,1)/Inv,'LineWidth',1.5);
    hold on
plot(time,100*u(1,:,1)/Inv,'--','LineWidth',1.5);
title(['I_t to preference shock'])
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,3,2)
plot(time,100*u_full(2,:,1)/n,'LineWidth',1.5);
    hold on
plot(time,100*u(2,:,1)/n,'--','LineWidth',1.5);
title('N_t to preference shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

%Compute C_t IRF
%Full information case
CtFull_D = k^ALFA*n^(1-ALFA)*X_full(2,:,1) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X_full(3,:,1) +...
    (1-ALFA)*k^ALFA*n^(-ALFA)*u_full(2,:,1) - u_full(1,:,1);
%RI case
CtRI_D =  k^ALFA*n^(1-ALFA)*X(2,:,1) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X(3,:,1) +...
    (1-ALFA)*k^ALFA*n^(-ALFA)*u(2,:,1) - u(1,:,1);

subplot(2,3,3)
plot(time,100*CtFull_D/c, 'LineWidth',1.5);
    hold on
plot(time,100*CtRI_D/c,'--','LineWidth',1.5);

title('C_t to preference shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,3,4)
plot(time,100*u_full(1,:,2)/Inv,'LineWidth',1.5);
    hold on
plot(time,100*u(1,:,2)/Inv,'--','LineWidth',1.5);
xlabel('time')
%legend('Full Info','RI')
title('I_t to TFP shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,3,5)
plot(time,100*u_full(2,:,2)/n,'LineWidth',1.5);
    hold on
plot(time,100*u(2,:,2)/n,'--','LineWidth',1.5);
xlabel('time')
%legend('Full Info','RI')
title('N_t to TFP shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

%Full information
CtFull_TFP = k^ALFA*n^(1-ALFA)*X_full(2,:,2) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X_full(3,:,2) +...
    (1-ALFA)*k^ALFA*n^(-ALFA)*u_full(2,:,2)- u_full(1,:,2);
%RI case
CtRI_TFP =  k^ALFA*n^(1-ALFA)*X(2,:,2) + ALFA*k^(ALFA-1)*n^(1-ALFA)*X(3,:,2) +...
    (1-ALFA)*k^ALFA*n^(-ALFA)*u(2,:,2) - u(1,:,2);

subplot(2,3,6)
plot(time,100*CtFull_TFP/c,'LineWidth',1.5);
    hold on
plot(time,100*CtRI_TFP/c,'--','LineWidth',1.5);
xlabel('time')
lgd = legend('Full Info','RI')
lgd.FontSize = 6;
title('C_t to TFP shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;



%Full information second moments
[sigyJ,sigxJ] = mom(-F,A-B*F, W,0)

%print -depsc FigRBC1.eps